package main

import (
	"embed"
	"errors"
	"io/fs"
	"log"
	"net/http"
	"os"
	"strconv"
	"time"

	"git.1e99.eu/1e99/passed/password"
	"git.1e99.eu/1e99/passed/route"
)

//go:embed static/*
var embedFS embed.FS

func run() error {
	storage, err := newStorage()
	if err != nil {
		return err
	}

	staticFS, err := newStaticFS()
	if err != nil {
		return err
	}

	var address string
	var logRequests bool
	var maxPasswordLength int
	env("PASSED_ADDRESS", &address, ":3000")
	env("PASSED_LOG_REQUESTS", &logRequests, "true")
	env("PASSED_MAX_LENGTH", &maxPasswordLength, "12288")

	mux := http.NewServeMux()
	handler := http.Handler(mux)

	mux.Handle("GET /", http.FileServerFS(staticFS))
	mux.Handle("POST /api/password", route.CreatePassword(storage, maxPasswordLength))
	mux.Handle("GET /api/password/{id}", route.GetPassword(storage))
	mux.Handle("HEAD /api/password/{id}", route.HasPassword(storage))

	if logRequests {
		handler = route.Logger(handler)
	}

	log.Printf("Listening on %s.", address)
	err = http.ListenAndServe(address, handler)
	if err != nil {
		return err
	}

	return nil
}

func newStaticFS() (sfs fs.FS, err error) {
	var fsType string
	env("PASSED_STATIC_TYPE", &fsType, "embed")

	switch fsType {
	case "embed":
		sfs, err = fs.Sub(embedFS, "static")
		return
	case "dir", "directory":
		var path string
		env("PASSED_STATIC_DIR_PATH", &path, "static")

		sfs = os.DirFS(path)
		return
	default:
		err = errors.New("unkown fs type")
		return
	}
}

func newStorage() (storage password.Storage, err error) {
	var storageType string
	var clearInterval int
	env("PASSED_STORE_TYPE", &storageType, "ram")
	env("PASSED_STORE_CLEAR_INTERVAL", &clearInterval, "30")

	switch storageType {
	case "ram":
		storage = password.NewRamStorage()
	case "dir", "directory":
		var path string
		env("PASSED_STORE_DIR_PATH", &path, "passwords")

		err = os.MkdirAll(path, os.ModePerm)
		if err != nil {
			return
		}

		storage = password.NewDirStorage(path)
	default:
		err = errors.New("unknown storage type")
		return
	}

	// Clear storage at regular intervals
	go func() {
		ticker := time.Tick(time.Duration(clearInterval) * time.Second)
		for {
			<-ticker

			err := storage.ClearExpired()
			if err != nil {
				log.Printf("Failed to clear expired passwords: %s", err)
				continue
			}

			log.Printf("Cleared expired passwords.")
		}
	}()

	return
}

func main() {
	err := run()
	if err != nil {
		log.Fatalf("%s", err)
	}
}

func env(name string, out any, def string) {
	raw := os.Getenv(name)
	if raw == "" {
		raw = def
		log.Printf("No \"%s\" provided, defaulting to \"%s\".", name, def)
	}

	switch value := out.(type) {
	case *int:
		i, err := strconv.ParseInt(raw, 10, 64)
		if err != nil {
			log.Printf("\"%s\" is not a number (\"%s\").", name, raw)
			return
		}

		*value = int(i)

	case *bool:
		switch raw {
		case "true", "TRUE", "1":
			*value = true
		case "false", "FALSE", "0":
			*value = false
		}

	case *string:
		*value = raw
	}
}
